/*
	Copyright 2009 Benjamin Vedder	vedder87@gmail.com
	
	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    */

package mario.dataCollections;

import java.util.LinkedList;
import java.util.List;

import mario.tools.Tile;

public class TileSet {
	private List<TileData> dataList = new LinkedList<TileData>();
	
	public TileSet() {
		
	}
	
	/**
	 * Constructor for raw tileset data.
	 * @param tileset
	 * An AVGA hardware-compatible tileset.
	 */
	public TileSet (char[] tileset, char[] colInfo) {
		int hbt = ((Tile.HEIGHT * Tile.WIDTH) / 2);
		char[][] tileArray = new char[tileset.length / hbt][hbt];

		int ind = 0;
		for (int i = 0; i < tileset.length; i += hbt) {
			char[] c = new char[hbt];

			for (int j = 0; j < hbt; j++) {
				
				c[j] = tileset[i + j];
			}

			tileArray[ind] = c;

			ind++;
		}
		
		ind = 0;
		for (char[] pix: tileArray) {
			TileData dta = new TileData(pix, ind);			
			// ----------------- Collision ----------------- //
			dta.collInfo.north = ((colInfo[ind] & 0x01) > 0);
			dta.collInfo.south = ((colInfo[ind] & 0x02) > 0);
			dta.collInfo.west = ((colInfo[ind] & 0x04) > 0);
			dta.collInfo.east = ((colInfo[ind] & 0x08) > 0);
			dta.collInfo.jump = ((colInfo[ind] & 0x10) > 0);
			dta.collInfo.water = ((colInfo[ind] & 0x20) > 0);
			// --------------------------------------------- //			
			
			addTile(dta);
			
			ind++;
			
			if (ind == Tile.MAX_TILES_PER_SET) {
				break;
			}
		}
	}
	
	/**
	 * Constructor to create tileset with empty tiles
	 * @param tiles
	 * the number of empty tiles
	 */
	public TileSet (int tiles) {
		int hbt = ((Tile.HEIGHT * Tile.WIDTH) / 2);
		char[][] tileArray = new char[tiles][hbt];

		int ind = 0;
		for (int i = 0; i < tiles * hbt; i += hbt) {
			char[] c = new char[hbt];

			for (int j = 0; j < hbt; j++) {
				
				c[j] = 0x00;
			}

			tileArray[ind] = c;

			ind++;
		}
		
		ind = 0;
		for (char[] pix: tileArray) {
			TileData dta = new TileData(pix, ind);
			ind++;
			
			addTile(dta);
			
			if (ind == Tile.MAX_TILES_PER_SET) {
				break;
			}
		}
	}
	
	public boolean addTile(TileData data) {
		if (dataList.size() > Tile.MAX_TILES_PER_SET) {
			return false;
		}
		
		data.index = dataList.size();
		dataList.add(data);
		
		return true;
	}
	
	public void addEmptyTiles(int n) {
		int a = dataList.size() + n;
		
		if (a > Tile.MAX_TILES_PER_SET) {
			a = Tile.MAX_TILES_PER_SET;
		}
		
		char[] pix = new char[(Tile.WIDTH * Tile.HEIGHT) / (8 / Tile.TILE_BPP)];
		
		for (char element: pix) {
			element = Tile.DEFAULT_COLOR;
		}
		
		for (int i = dataList.size();i < a;i++) {
			TileData td = new TileData(pix, i);
			dataList.add(td);
		}
	}
	
	public void removeTiles(int n) {
		for (int i = 0;i < n;i++) {
			dataList.remove(dataList.size() - 1);
		}
	}
	
	public TileData getTile(int index) {
		return dataList.get(index);
	}
	
	public List<TileData> getTiles() {
		return dataList;
	}
	
	public int getLength() {
		return dataList.size();
	}
	
	public char[] getRawData() {
		int hbt = ((Tile.HEIGHT * Tile.WIDTH) / 2);
		char[] d = new char[dataList.size() * hbt];
		
		int ind = 0;
		for (TileData element: dataList) {
			for (char element2: element.pixels) {
				d[ind] = element2;
				ind++;
			}
		}
		
		return d;
	}
	
	/**
	 * Get data for the collision detector.
	 * @return
	 * Array with collision data. Each tile index from the current tileset
	 * is the same as the corresponding collision byte in the collision array.
	 */
	public char[] getRawCollisionData() {
		char[] data = new char[dataList.size()];
		
		int ind = 0;
		for (TileData element: dataList) {
			char c = 0x00;
			
			if (element.collInfo.north) {
				c = (char)(c | 0x01);
			}
			if (element.collInfo.south) {
				c = (char)(c | 0x02);
			}
			if (element.collInfo.west) {
				c = (char)(c | 0x04);
			}
			if (element.collInfo.east) {
				c = (char)(c | 0x08);
			}
			if (element.collInfo.jump) {
				c = (char)(c | 0x10);
			}
			if (element.collInfo.water) {
				c = (char)(c | 0x20);
			}

			data[ind] = c;
			ind++;
		}
		
		return data;
	}
}














